//
// Copyright (C) 2005 Mirko Nasato <mirko@artofsolving.com>
//
// Distributable under the terms of the LGPL license.
// See http://www.gnu.org/copyleft/lesser.html
//
package net.sf.davcache;

import java.io.FileNotFoundException;
import java.io.InputStream;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.jboss.cache.PropertyConfigurator;
import org.jboss.cache.TreeCache;

/**
 * A ContextListener that starts/stops a TreeCache and a {@link net.sf.davcache.TreeCacheStore} instances.
 * <p>
 * An alternative to the JBoss service MBean that works with any servlet container.
 *
 * @author Mirko Nasato
 */
public class DavCacheContextListener implements ServletContextListener
{
   private static final String CONFIGURATION_FILE = "/WEB-INF/davcache-service.xml";

   public void contextInitialized(ServletContextEvent event)
   {
      ServletContext context = event.getServletContext();
      try
      {
         InputStream input = context.getResourceAsStream(CONFIGURATION_FILE);
         if (input == null)
            throw new FileNotFoundException("configuration file not found: " + CONFIGURATION_FILE);

         TreeCache treeCache = new TreeCache();
         PropertyConfigurator configurator = new PropertyConfigurator();
         configurator.configure(treeCache, input);
         input.close();
         context.setAttribute(TreeCache.class.getName(), treeCache);
         treeCache.createService();
         treeCache.startService();
         
         TreeCacheStoreMBean treeCacheStore = new TreeCacheStore();
         treeCacheStore.setTreeCache(treeCache);
         context.setAttribute(TreeCacheStore.class.getName(), treeCacheStore);
         treeCacheStore.start();
      }
      catch (Exception e)
      {
         throw new RuntimeException("failed to start TreeCache and TreeCacheStore", e);
      }
   }

   public void contextDestroyed(ServletContextEvent event)
   {
      TreeCacheStoreMBean treeCacheStore = (TreeCacheStoreMBean) event.getServletContext().getAttribute(TreeCacheStore.class.getName());
      if (treeCacheStore != null)
      {
         treeCacheStore.stop();
      }
      
      TreeCache treeCache = (TreeCache) event.getServletContext().getAttribute(TreeCache.class.getName());
      if (treeCache != null)
      {
         treeCache.stopService();
         treeCache.destroyService();
      }
   }
}
