//
// Copyright (C) 2005 Mirko Nasato <mirko@artofsolving.com>
//
// Distributable under the terms of the LGPL license.
// See http://www.gnu.org/copyleft/lesser.html
//
package net.sf.davcache;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class FileTypes
{

   private static final String DEFAULT_TYPE = "application/octect-stream";

   private static final String PROPERTIES_FILE = "fileTypes.properties";

   private static final Log log = LogFactory.getLog(FileTypes.class);

   private Properties fileTypes = new Properties();

   public FileTypes()
   {
      loadProperties();
   }

   public String getContentType(String fileName)
   {
      return getContentTypeByFileExtension(getFileExtension(fileName));
   }

   public String getContentTypeByFileExtension(String fileExtension)
   {
      String contentType = fileTypes.getProperty(fileExtension);
      if (contentType == null)
         return DEFAULT_TYPE;
      return contentType;
   }

   public static String getFileExtension(String fileName)
   {
      int dot = fileName.lastIndexOf('.');
      if (dot == -1)
      {
         return "";
      }
      return fileName.substring(dot + 1);
   }

   private void loadProperties()
   {
      InputStream input = getClass().getResourceAsStream(PROPERTIES_FILE);
      if (input == null)
      {
         log.error("file types configuration not found in classpath: " + PROPERTIES_FILE);
         return;
      }
      log.debug("loading file type properties");
      try
      {
         fileTypes.load(input);
         input.close();
      }
      catch (IOException e)
      {
         log.error("failed to load " + PROPERTIES_FILE, e);
      }
   }
}
