//
// Copyright (C) 2005 Mirko Nasato <mirko@artofsolving.com>
//
// Distributable under the terms of the LGPL license.
// See http://www.gnu.org/copyleft/lesser.html
//
package net.sf.davcache;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.QName;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

// we're always returning '409 Conflict' here as all available properties are considered read-only
public class PropPatchResponse
{
   public static final int CODE_MULTI_STATUS = 207;

   private static final String STATUS_CONFLICT = "HTTP/1.1 409 Conflict";

   private List/*QName*/ requestedProperties = new ArrayList();

   private Document document;

   private Element multistatus;

   public PropPatchResponse(HttpServletRequest request) throws IOException, DocumentException
   {
      parseRequest(request);
      document = DocumentHelper.createDocument();
      multistatus = document.addElement(DavXmlUtils.toDavQName("multistatus"));
   }

   public void setResource(String path)
   {
      Element response = DavXmlUtils.addDavElement(multistatus, "response");
      DavXmlUtils.addDavElement(response, "href", path);
      Element propStat = DavXmlUtils.addDavElement(response, "propstat");
      Element prop = DavXmlUtils.addDavElement(propStat, "prop");
      for (Iterator iterator = requestedProperties.iterator(); iterator.hasNext();)
      {
         QName qProperty = (QName) iterator.next();
         if ("DAV:".equals(qProperty.getNamespaceURI()))
         {
            // use "D:" prefix as we do with all other DAV properties 
            qProperty = DavXmlUtils.toDavQName(qProperty.getName());
         }
         prop.addElement(qProperty);
      }
      DavXmlUtils.addDavElement(propStat, "status", STATUS_CONFLICT);
      DavXmlUtils.addDavElement(propStat, "responsedescription", "All properties are read-only.");
   }

   public void send(HttpServletResponse response) throws IOException
   {
      response.setStatus(CODE_MULTI_STATUS);
      response.setContentType("text/xml;charset=UTF-8");
      XMLWriter writer = new XMLWriter(response.getOutputStream(), new OutputFormat("    ", true));
      writer.write(document);
   }

   private void parseRequest(HttpServletRequest request) throws IOException, DocumentException
   {
      SAXReader reader = new SAXReader();
      Document document = reader.read(request.getInputStream());
      Element propertyUpdate = document.getRootElement();
      if (propertyUpdate == null)
      {
         throw new DocumentException("no root element");
      }
      if (!propertyUpdate.getName().equals("propertyupdate"))
      {
         throw new DocumentException("root element is not 'propertyupdate'");
      }
      for (Iterator iterator = propertyUpdate.elementIterator(); iterator.hasNext();)
      {
         Element action = (Element) iterator.next();
         if (!("set".equals(action.getName()) || "remove".equals(action.getName())))
         {
            throw new DocumentException("unknown element inside 'propertyupdate': " + action.getName());
         }
         if (action.elements().size() != 1)
         {
            throw new DocumentException("multiple elements inside '" + action.getName()
                  + "'; expecting a single 'prop'");
         }
         Element prop = (Element) action.elements().get(0);
         for (Iterator propIterator = prop.elementIterator(); propIterator.hasNext();)
         {
            Element property = (Element) propIterator.next();
            requestedProperties.add(property.getQName());
         }
      }
   }

}
